"use strict";
const hrtime_now_1 = require("hrtime-now");
const path_1 = require("path");
const typescript_plugin_util_1 = require("typescript-plugin-util");
const typescript_plugins_text_based_user_interaction_1 = require("typescript-plugins-text-based-user-interaction");
const moveDeclaration_1 = require("./moveDeclaration");
const PLUGIN_NAME = 'typescript-plugin-move-declaration';
const REFACTOR_ACTION_NAME = `${PLUGIN_NAME}-refactor-action`;
const interactionTool = typescript_plugins_text_based_user_interaction_1.create({
    prefix: '&%&%',
    actions: [
        {
            name: 'moveDeclarationNamed',
            args: ['declarationName', 'dest'],
            print: (action) => `Move "${action.args.declarationName}" to file ${action.args.dest}`,
            snippet: 'moveDeclarationNamed(\'SomeClass\', \'../other/file.ts\')'
        }
    ]
});
let selectedAction;
function getApplicableRefactors(fileName, positionOrRange, userPreferences) {
    const t0 = hrtime_now_1.now();
    const refactors = info.languageService.getApplicableRefactors(fileName, positionOrRange, userPreferences) || [];
    const program = info.languageService.getProgram();
    const sourceFile = program.getSourceFile(fileName);
    if (!sourceFile) {
        return refactors;
    }
    const actions = interactionTool.findActions(sourceFile.getText());
    if (!actions || actions.length === 0) {
        return refactors;
    }
    selectedAction = actions[0];
    const refactorActions = [{ name: REFACTOR_ACTION_NAME, description: selectedAction.print(selectedAction) }];
    refactors.push({
        name: `${PLUGIN_NAME}-refactor-info`,
        description: 'move-declaration',
        actions: refactorActions
    });
    info.project.projectService.logger.info(`${PLUGIN_NAME} getApplicableRefactors took ${(hrtime_now_1.now() - t0) / 1000000}`);
    return refactors;
}
function getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName, userPreferences) {
    const t0 = hrtime_now_1.now();
    const refactors = info.languageService.getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName, userPreferences);
    if (actionName !== REFACTOR_ACTION_NAME) {
        return refactors;
    }
    try {
        // TODO: could we maintain a simple-ast Project in a variable and next time just refresh it so is faster ?
        // perhaps with this: simpleProject.getSourceFiles().forEach(f=>f.refreshFromFileSystemSync())
        const simpleProject = typescript_plugin_util_1.createSimpleASTProject(info.project);
        const sourceFile = simpleProject.getSourceFileOrThrow(fileName);
        let dest = path_1.isAbsolute(selectedAction.args.dest) ? selectedAction.args.dest :
            path_1.join(path_1.dirname(fileName), selectedAction.args.dest);
        const targetFile = simpleProject.getSourceFile(dest);
        info.project.projectService.logger.info(`${PLUGIN_NAME} getEditsForRefactor moveDeclarationNamed ${selectedAction.args.declarationName}, ${sourceFile.getFilePath()}, ${targetFile.getFilePath()}`);
        moveDeclaration_1.moveDeclarationNamed(selectedAction.args.declarationName, sourceFile, simpleProject, targetFile);
        info.languageService.getEmitOutput(sourceFile.getFilePath());
        info.languageService.getEmitOutput(targetFile.getFilePath());
        simpleProject.saveSync();
    }
    catch (error) {
        info.project.projectService.logger.info(`${PLUGIN_NAME} getEditsForRefactor error  ${selectedAction.name} ${error + ' - ' + error.stack}`);
        return refactors;
    }
    info.project.projectService.logger.info(`${PLUGIN_NAME} getEditsForRefactor ${selectedAction.name} took  ${(hrtime_now_1.now() - t0) / 1000000}`);
}
function getCompletionsAtPosition(fileName, position, options) {
    const prior = info.languageService.getCompletionsAtPosition(fileName, position, options);
    if (prior) {
        prior.entries = prior.entries.concat(interactionTool.getCompletionsAtPosition(fileName, position, options));
    }
    return prior;
}
;
let ts;
let info;
const pluginDefinition = {
    getApplicableRefactors, getEditsForRefactor, getCompletionsAtPosition
};
module.exports = typescript_plugin_util_1.getPluginCreate(pluginDefinition, (modules, anInfo) => {
    ts = modules.typescript;
    info = anInfo;
    info.project.projectService.logger.info(`${PLUGIN_NAME} created`);
});
//# sourceMappingURL=index.js.map